package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import de.hybris.platform.cronjob.model.CronJobModel;
import java.util.Calendar;
import com.bpcl.lpg.facades.ym.data.LpgYMV2ResponseData;
import org.apache.commons.lang3.StringUtils;
import java.text.DateFormat;
import java.util.Objects;
import com.bpcl.lpg.core.model.CashMemoFileTrackerModel;
import java.util.Date;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import com.bpcl.lpg.core.dao.LpgDeliveryConfirmationDao;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.core.model.BpclCronjobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class LpgCashMemoNotReceivedAlertJob extends AbstractJobPerformable<BpclCronjobModel>
{
    private static final Logger LOG;
    private static final String EMAIL_LINE_1 = "Dear Team,";
    private static final String EMAIL_LINE_2 = "Please note that the last cashmemo file with name “{fileName}” had been received on STFP at {time} hrs.";
    private static final String EMAIL_LINE_3 = "Since then no new cashmemo file is received on SFTP.";
    private static final String EMAIL_LINE_4 = "This is for information and action at your end pls.";
    private static final String EMAIL_LINE_5 = "Regards,";
    private static final String EMAIL_LINE_6 = "LPG Operator App.";
    private static final String CASH_MEMO_FILE_TIME = "{time}";
    private static final String CASH_MEMO_FILE_NAME = "{fileName}";
    public static final String EMPTY = "";
    private ConfigurationService configurationService;
    private NotificationEngineService notificationEngineService;
    private LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final BpclCronjobModel cronjob) {
        final Date startDate = this.populateStartDate(cronjob);
        LpgCashMemoNotReceivedAlertJob.LOG.info("Date ranges starts from : {}", startDate);
        final Date endDate = this.populateEndDate(cronjob);
        LpgCashMemoNotReceivedAlertJob.LOG.info("Filter End Date : {}", endDate);
        try {
            final CashMemoFileTrackerModel lastCashMemoFile = this.lpgDeliveryConfirmationDao.getCashMemoFilesData(startDate, endDate);
            LpgCashMemoNotReceivedAlertJob.LOG.info("Last Cash Memo File : {}", lastCashMemoFile.getCashMemoFileName());
            final boolean isFileReceived = this.checkLastFileTimeDifference(lastCashMemoFile);
            if (!isFileReceived) {
                return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
            }
            final String subject = this.prepareSubject(lastCashMemoFile.getProcessStartTimeStamp());
            final StringBuilder emailBody = this.prepareEmailBody(lastCashMemoFile.getCashMemoFileName(), lastCashMemoFile.getProcessStartTimeStamp());
            if (this.sendEmail(subject, emailBody.toString())) {
                return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
            }
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.FINISHED);
        }
        catch (final Exception ex) {
            LpgCashMemoNotReceivedAlertJob.LOG.error(ex.getMessage(), ex.getCause());
            final String body = MessageFormat.format("Cash Memo Not Received cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Cash Memo Not Received Job Alert", body, this.configurationService.getConfiguration().getString("lpg.cash.memo.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    private Boolean checkLastFileTimeDifference(final CashMemoFileTrackerModel cashMemoFile) {
        final Date currentDate = new Date();
        LpgCashMemoNotReceivedAlertJob.LOG.info("Current Date and Time : {}", currentDate);
        final Date lastFileInitDate = cashMemoFile.getProcessStartTimeStamp();
        LpgCashMemoNotReceivedAlertJob.LOG.info("Last Cash Memo File INIT time : {}", lastFileInitDate);
        final long timeDifference = this.configurationService.getConfiguration().getLong("cash.memo.file.time.difference", 1200000L);
        if (currentDate.getTime() - lastFileInitDate.getTime() > timeDifference) {
            LpgCashMemoNotReceivedAlertJob.LOG.info("NO Cash Memo file received between {} and {}", currentDate, lastFileInitDate);
            return true;
        }
        LpgCashMemoNotReceivedAlertJob.LOG.info("Last Cash Memo file {} received at {}", cashMemoFile.getCashMemoFileName(), lastFileInitDate);
        return false;
    }
    
    private String prepareSubject(final Date lastFileInitDate) {
        final StringBuilder sub = new StringBuilder(this.configurationService.getConfiguration().getString("lpg.cash.memo.not.received.subject"));
        final String lastFileReceivedTime = dateToString(lastFileInitDate, "dd-MMM-yyyy h:mm a");
        sub.append(" after ").append(lastFileReceivedTime);
        return sub.toString();
    }
    
    public static String dateToString(final Date date, final String targetPattern) {
        final DateFormat dateFormat = BPCLDateUtils.getSimpleDateFormat(targetPattern);
        return Objects.isNull(date) ? "" : dateFormat.format(date);
    }
    
    private boolean sendEmail(final String emailSubjectLine, final String emailBody) {
        final String userId = this.configurationService.getConfiguration().getString("lpg.cash.memo.file.not.received.receiver.id");
        if (StringUtils.isBlank((CharSequence)userId)) {
            LpgCashMemoNotReceivedAlertJob.LOG.error("No recipients mentioned, Hence email couldn't be triggered");
            return Boolean.FALSE;
        }
        final LpgYMV2ResponseData lpgYMV2ResponseData = this.lpgV2NotificationEngineService.sendEmail(emailSubjectLine, emailBody, userId);
        if (Objects.isNull(lpgYMV2ResponseData) || StringUtils.isBlank((CharSequence)lpgYMV2ResponseData.getMsgId())) {
            LpgCashMemoNotReceivedAlertJob.LOG.error("Email wasn't sent");
            return Boolean.FALSE;
        }
        LpgCashMemoNotReceivedAlertJob.LOG.info("Email sent successfully");
        return Boolean.TRUE;
    }
    
    private StringBuilder prepareEmailBody(final String cashMemoFileName, final Date fileTime) {
        final String cashMemoFileTime = dateToString(fileTime, "HH:mm:ss");
        final StringBuilder emailBody = new StringBuilder("<html><body>").append("<p>");
        final String emailString = "Dear Team,<br/><br/>Please note that the last cashmemo file with name “{fileName}” had been received on STFP at {time} hrs.<br/><br/>Since then no new cashmemo file is received on SFTP.<br/><br/>This is for information and action at your end pls.<br/><br/>Regards,<br/>LPG Operator App.";
        emailBody.append(emailString.replace("{fileName}", cashMemoFileName).replace("{time}", cashMemoFileTime)).append("</p>").append("</body></html>");
        return emailBody;
    }
    
    private Date populateStartDate(final BpclCronjobModel cronjob) {
        Date startDate = cronjob.getFilterStartDate();
        if (startDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.add(10, -this.configurationService.getConfiguration().getInt("lpg.cash.memo.file.not.received.period"));
            startDate = cal.getTime();
        }
        return startDate;
    }
    
    private Date populateEndDate(final BpclCronjobModel cronjob) {
        Date endDate = cronjob.getFilterEndDate();
        if (endDate == null) {
            final Calendar cal = Calendar.getInstance();
            endDate = cal.getTime();
        }
        return endDate;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setLpgDeliveryConfirmationDao(final LpgDeliveryConfirmationDao lpgDeliveryConfirmationDao) {
        this.lpgDeliveryConfirmationDao = lpgDeliveryConfirmationDao;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgCashMemoNotReceivedAlertJob.class);
    }
}
